///////////////////////////////////////////////////////////////////////////////
// FileRecord.h
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <vector>

#include "Definitions.h"

class VFileRecord
{
public:
	enum EDataType		{ EUndefined = 0, EBinary, EText, EShort, ELong, EFloat, EDouble, EBoolean, EStructure };

private:
	bool	m_bEdited;	// Flag to determine the need for rewriting
	SSHORT	m_Length;
protected:
	BYTE *	m_RawData;	// Actual data from the file

	void SetLength(SSHORT a_Len);
	
public:
	VFileRecord();
	VFileRecord(BYTE * m_RawData, SSHORT a_Len);
	virtual ~VFileRecord();

	const BYTE * GetDataBuffer() { return m_RawData; }

	// The following functions, GetLength and CalculateLength, differ in the respect that
	// GetLength returns the length of the byte sequence which was read in when the record
	// was created. CalculateLength must be overloaded in the derived class since it
	// adds up the number of bytes necessary to write out the record in its current state,
	// i.e. taking all record specific fields into account. These functions are used to 
	// determine if the record is shrinking or growing.

	int GetLength() const { return m_Length; }
	virtual int CalculateLength() const = 0;

	// BuildRecordFromBytes takes the raw data from the disk which are stored in its
	// buffer, interprets it and builds a record. Its sister function BuildBytesFromRecord
	// does precisely the reverse - it prepares a list of bytes (for writing) from the 
	// data within the record.

	virtual void BuildRecordFromBytes() = 0;
	virtual SSHORT BuildBytesFromRecord() = 0;

//	void ByteCopy(BYTE * a_pDestBuf, const BYTE * a_pSrcBuf, int a_NumToCopy);

	void SetEdited() { m_bEdited = true; }
};

