#pragma once

#include <string>
#include <maya/MFnAnimCurve.h>
#include "FileDynamicRecord.h"
#include <maya/MDistance.h>
#include <maya/MEulerRotation.h>

class VMayaBody
{
public:
	enum EDof		{ EUndefDof = -1, ERx = 0, ERy, ERz, ETx, ETy, ETz };
	enum EOrient	{ EY_Up, EZ_Up };
	enum EBodyType	{ EUndefType = -1, ELocal, EGlobal };

private:
	std::string m_strBodyName;
	int m_DatagroupID;
	int m_DofIndices[6];	// tx ty tz ax ay az - indices to the components of trans/rot
							// within dynamic record.
	
	MFnAnimCurve * m_pAnimCurves[6];

	MEulerRotation m_LastEuler;
	bool m_bFirstEuler;

	EOrient m_WorldUp;
	
	void ConvertAngleAxisToQuat(double a_q[4], const double a_r[3]);
	void ConvertAngleAxisToMatrix(double a_M[4][4], const double a_r[3]);

	float m_MayaFrameRate;
	MDistance::Unit m_DistUnit;
	
	void ConvertToInternalUnits(double & a_Value);
	void ConvertToInternalUnits(float & a_Value);

public:
	VMayaBody(std::string a_strBodyName, int a_GroupID, EOrient a_Orient = EZ_Up);
	virtual ~VMayaBody();

	EDof ConvertDofSuffix(std::string a_strPre, std::string & a_rstrPost);

	void SetDofIndex(EDof a_eComponent, int a_Index);
	void SetAnimCurve(EDof a_eComponent, MFnAnimCurve * a_pCurve);

	void SetKeyframe(VFileDynamicRecord & a_rRecord, float a_FrameRate);

	std::string Name() { return m_strBodyName; }
};

static const double ConvSmallNum = 1e-15;

#define V_POSX			"<P-X>"
#define V_POSY			"<P-Y>"
#define V_POSZ			"<P-Z>"
#define V_ROTX			"<A-X>"
#define V_ROTY			"<A-Y>"
#define V_ROTZ			"<A-Z>"
#define V_TRANSX		"<T-X>"
#define V_TRANSY		"<T-Y>"
#define V_TRANSZ		"<T-Z>"
#define V_LOCALROTX		"<a-X>"
#define V_LOCALROTY		"<a-Y>"
#define V_LOCALROTZ		"<a-Z>"
#define V_LOCALTRANSX	"<t-X>"
#define V_LOCALTRANSY	"<t-Y>"
#define V_LOCALTRANSZ	"<t-Z>"

#define MAYA_POSX		"tx"
#define MAYA_POSY		"ty"
#define MAYA_POSZ		"tz"
#define MAYA_ROTX		"rx"
#define MAYA_ROTY		"ry"
#define MAYA_ROTZ		"rz"

std::string TrimLeft(std::string a_strLabel);
std::string TrimRight(std::string a_strLabel);
float GetMayaFrameRate();
