#include "FileDynamicRecord.h"

VFileDynamicRecord::VFileDynamicRecord(BYTE * a_RawData, short a_sRecLen) : VFileRecord(a_RawData, a_sRecLen)
{
	m_DatagroupID = -1;
	m_FrameNumber = -1;
//	m_pDatagroupRec = NULL;
	m_pDofValues = &a_RawData[sizeof(SSHORT) + sizeof(SLONG)];	// The data starts after the rec len and frame no.
	m_DofDataLen = 0;
}

VFileDynamicRecord::VFileDynamicRecord(VFileDatagroupRecord * a_pDGRecord) 
{
	assert(a_pDGRecord != NULL);

//	m_pDatagroupRec = a_pDGRecord;
	m_DatagroupID = a_pDGRecord->GetGroupID();
	m_FrameNumber = -1;

	m_DofDataLen = a_pDGRecord->GetDataWidth() * a_pDGRecord->GetNumEntries();
	
	int RemainingLen = sizeof(SSHORT) + sizeof(SLONG);	// For the datagroupID and frame number
	SetLength(m_DofDataLen + RemainingLen);

	m_RawData = new BYTE[m_DofDataLen + RemainingLen];

	assert(m_RawData);

	if (m_RawData != NULL)
		m_pDofValues = &m_RawData[RemainingLen]; // The dof data starts immediately after the ID and frame no.
}

VFileDynamicRecord::~VFileDynamicRecord()
{
	//VFileRecord::~VFileRecord();
}

int VFileDynamicRecord::CalculateLength() const
{
	return sizeof(SSHORT) +		// Datagroup ID
		   sizeof(SLONG) +		// frame number
		   m_DofDataLen;		// Width of remaining data
}

void VFileDynamicRecord::BuildRecordFromBytes()
{
	assert(m_RawData != NULL);

	if (m_RawData != NULL)
	{
		int ByteIndex = 0;

		memcpy(&m_DatagroupID, &m_RawData[ByteIndex], sizeof(SSHORT));
		ByteIndex += sizeof(SSHORT);

		memcpy(&m_FrameNumber, &m_RawData[ByteIndex], sizeof(SLONG));
		ByteIndex += sizeof(SLONG);

		m_DofDataLen = GetLength() - ByteIndex;

		m_pDofValues = &m_RawData[ByteIndex];
	}
}

SSHORT VFileDynamicRecord::BuildBytesFromRecord()
{
	SSHORT OldSize = GetLength();
	SSHORT NewSize = CalculateLength();

	if (OldSize != NewSize)
	{	
		assert(false);
		delete [] m_RawData;		// This is shit of course

		m_RawData = new BYTE[NewSize];
	}

	assert(m_RawData);

	if (m_RawData)
	{
		int ByteIndex = 0;

		// Got the memory... proceed

		memcpy(&m_RawData[ByteIndex], &m_DatagroupID, sizeof(SSHORT));
		ByteIndex += sizeof(SSHORT);

		memcpy(&m_RawData[ByteIndex], &m_FrameNumber, sizeof(SLONG));
		ByteIndex += sizeof(SLONG);

		SetLength(NewSize);
	}
	else
	{
		// Out of memory

		SetLength(0);
		NewSize = 0;
	}

	return NewSize;
}

void VFileDynamicRecord::SetDatagroupID(SSHORT a_ID)
{
	assert(a_ID >= 0);

	m_DatagroupID = a_ID;
}

void VFileDynamicRecord::SetFrameNumber(SLONG a_FrameNum)
{
	assert(a_FrameNum > 0);

	m_FrameNumber = a_FrameNum;
}

bool VFileDynamicRecord::GetDofValAsShort(int a_Index, SSHORT & a_rVal) const
{
	SSHORT * pShorts = (SSHORT *)m_pDofValues;
	
	a_rVal = pShorts[a_Index];

	return pShorts[a_Index] != (SSHORT)0xFFFF;
}

bool VFileDynamicRecord::GetDofValAsLong(int a_Index, SLONG & a_rVal) const
{
	SLONG * pLongs = (SLONG *)m_pDofValues;

	a_rVal = pLongs[a_Index];

	return pLongs[a_Index] != (SLONG)0xFFFFFFFF;
}

bool VFileDynamicRecord::GetDofValAsFloat(int a_Index, float & a_rVal) const
{
	float * pFloats = (float *)m_pDofValues;

	a_rVal = pFloats[a_Index];

	return pFloats[a_Index] != (float)0xFFFFFFFF;
}

bool VFileDynamicRecord::GetDofValAsDouble(int a_Index, double & a_rVal) const
{
	double * pDoubles = (double *)m_pDofValues;

	a_rVal = pDoubles[a_Index];

	return pDoubles[a_Index] != (double)0xFFFFFFFFFFFFFFFFLL;
}

bool VFileDynamicRecord::GetDofValAsBinary(int a_Index, int a_NumBytes) const
{
	assert(false);

//	a_rVal = m_pDofValues;

	return false;
}

bool VFileDynamicRecord::GetDofValAsBool(int a_Index, bool & a_rVal) const
{
	a_rVal = (m_pDofValues[a_Index] != 0);

	return m_pDofValues[a_Index] != 0xFF;
}

void VFileDynamicRecord::SetDofValAsShort(int a_Index, SSHORT a_Val)
{
	SSHORT * pShorts = (SSHORT *)m_pDofValues;

	pShorts[a_Index] = a_Val;
}

void VFileDynamicRecord::SetDofValAsLong(int a_Index, SLONG a_Val)
{
	SLONG * pLongs = (SLONG *)m_pDofValues;

	pLongs[a_Index] = a_Val;
}

void VFileDynamicRecord::SetDofValAsFloat(int a_Index, float a_Val)
{
	float * pFloats = (float *)m_pDofValues;

	pFloats[a_Index] = a_Val;
}

void VFileDynamicRecord::SetDofValAsDouble(int a_Index, double a_Val)
{
	double * pDoubles = (double *)m_pDofValues;

	pDoubles[a_Index] = a_Val;
}

void VFileDynamicRecord::SetDofValAsBinary(int a_Index, BYTE * a_Val)
{
	assert(false);
}

void VFileDynamicRecord::SetDofValAsBool(int a_Index, bool a_Val)
{
	m_pDofValues[a_Index] = a_Val? 1 : 0;
}

bool VFileDynamicRecord::IsValidIndex(int a_Index)
{
	assert(false);
	return false;
}

